# -*- coding: iso-8859-15 -*-
import os
import Numeric

import cdms
import MV
import vcs

# filtre un flichier cr par cdms avec un ncdump, puis un ncgen.
def filtre_dump (fichier):
    os.spawnlp(os.P_WAIT,"filtre_dump","filtre_dump",fichier,fichier)


# calcul le tableau d'index d'un axe de points de terre
# (ie kindex2D[i,j] = land[kij]),
# ainsi que les indices inverss
# (ie ( iland[ kindex2D[i,j]],jland[kindex2D[i,j]] ) = ( i,j )).
# l'algorithme est celui utilis dans ORCHIDEE.
def calcul_kindex(kindex2D,iland,jland,axisLand,iim,jjm):
    kij=0
    for ki in axisLand:
        j = int((ki-1)/iim)
        i = int(((ki-1) - j*iim))
        kindex2D[i,j] = int(kij)
        iland[kij] = i
        jland[kij] = j
        #kindex2Dt[jjm-1-j,i] = kij
        kij = kij+1
    return

# calcul le vecteur des points de terre  partir du tableau des index
# (ie land[kij] = kindex2D[i,j])
# l'algorithme est celui utilis dans ORCHIDEE.
def calcul_axisLand(kindex2D,iim,jjm,missing_value):
    axisLand = []
    kij=0
    for j in range(jjm):
        for i in range(iim):
            if (kindex2D[i,j] < missing_value):
                ki = int(j*iim + i + 1)
                axisLand.append(ki)
                # vrification : 
                if (j != (ki-1)/iim or i != ((ki-1) - j*iim)):
                    print "Erreur calcul_axisLand au ",kij,"-ime point de terre pour (i,j) = ",i,j,"  et land_i = ",ki
		kij = kij+1
    return(MV.array(axisLand))


# Converti une variable d'axes [t,z,land] (avec land les points de terre)
# en une variable 2D (lat,lon)
def conversion2D(VarIn, kindex, nbindex, iim, jjm, missing_value):
    VarOut = missing_value*MV.ones((jjm,iim))
    for j in range(jjm):
	for i in range(iim):
            if (kindex[i,j] < missing_value):
                #print i,j,kindex[j,i],(jjm-1)-j
                VarOut[j,i] = VarIn[kindex[i,j]]
    return(VarOut)

# Converti une variable 2D (lat,lon) en
# une variable d'axe land (les points de terre).
def conversion2Dinv(VarIn, kindex, nbindex, iim, jjm, missing_value):
    VarOut = missing_value*MV.ones(nbindex)
    ii=0
    for j in range(jjm):
	for i in range(iim):
		if (kindex[i,j] < missing_value):
                    VarOut[ii] = VarIn[j,i]
                    ii=ii+1
    return(VarOut)

# Pour les graphiques : mme fonction, avec la latitude inverse.
def conversion2DGr(VarIn, kindex, nbindex, iim, jjm, missing_value):
    VarOutGr = missing_value*Numeric.ones((jjm,iim))
    for j in range(jjm):
	for i in range(iim):
		if (kindex[i,j] < missing_value):
                        #print i,j,kindex[j,i],(jjm-1)-j
			VarOutGr[(jjm-1)-j,i] = VarIn[kindex[i,j]]
    return(VarOutGr)

# Pour les graphiques : inverse la latitude d'une variable 2D
def inverse2DGr(VarIn, kindex, nbindex, iim, jjm, missing_value):
    VarOutGr = missing_value*Numeric.ones((jjm,iim))
    for j in range(jjm):
	for i in range(iim):
		if (kindex[i,j] < missing_value):
                    VarOutGr[(jjm-1)-j,i] = VarIn[j,i]
    return(VarOutGr)

# Calcul l'chelle iso d'une variable Var, dans un graphique grd, avec une valeur missing_value.
# On dcale les grandeurs min/max de decal % 
def errlevGr(Var, grd, decal, missing_value):    
    minv = cdms.MV.minimum(Var)
    maxv = cdms.MV.maximum(Var)
    if (minv > 0):
        minv = minv*(1-decal/100.)
    else:
        minv = minv*(1+decal/100.)
    if (maxv > 0):
        maxv = maxv*(1+decal/100.)
    else:
        maxv = maxv*(1-decal/100.)
    diffv = (maxv - minv)/10. 
    errlevs = vcs.mkevenlevels(minv,maxv,12)
    errlevs.append(missing_value)
    errlevs.insert(0,missing_value)
    cols = vcs.getcolors(errlevs)

    return(errlevs, cols)



# The input VarIn is a land variable.
# The output variable is 2D ( [jjm,iim] or [jmin:(jmax+1),imin:(imax+1)] )
#            and if calcland==1, it returns too :
#                zoomkindex, zoomiim, zoomjjm, nlandz,
#                invindex, ifenetre =(imin,imax,jmin,jmax)
def zoom_land(VarIn, kindex, nbindex, lat, lon, iim, jjm, limits, missing_value, calcland=1):
# where limits is in the order : [limit_W, limit_E, limit_N, limit_S]
    VarOut = missing_value*Numeric.ones((jjm,iim))
    # inverse of kindex
    invindex = Numeric.zeros((nbindex,2))
    imin = iim+1
    imax = -1
    jmin = jjm+1
    jmax = -1
    for j in range(jjm):
	for i in range(iim):
		if (kindex[i,j] < missing_value):
			if ( limits[0] <= lon[j,i] and lon[j,i] <= limits[1]
			     and limits[3] <= lat[j,i] and lat[j,i] <= limits[2] ):
				imin = min(imin, i)
				imax = max(imax, i)
				jmin = min(jmin, j)
				jmax = max(jmax, j)
                                invindex[kindex[i,j],0]=j
                                invindex[kindex[i,j],1]=i
				VarOut[j,i] = VarIn[kindex[i,j]]
    zoomiim = imax - imin +1
    zoomjjm = jmax - jmin +1
    #print imin, imax, jmin, jmax
    #print zoomiim, zoomjjm
    # pourquoi le +1 ???? (python ??) => ncessaire en tout cas.
    #  cause des lat/lon qui doivent anglober la zone zoome
    VarOutZoom = VarOut[jmin:(jmax+1),imin:(imax+1)]
    if (calcland==1):
        zoomkindex = Numeric.ones((zoomiim,zoomjjm))*int(missing_value)
        nlandz=0
        for j in range(jjm):
            for i in range(iim):
		if (kindex[i,j] < missing_value):
                    if ( limits[0] <= lon[j,i] and lon[j,i] <= limits[1]
                         and limits[3] <= lat[j,i] and lat[j,i] <= limits[2] ):
                        zoomkindex[i-imin, j-jmin] = kindex[i,j]
                        nlandz=nlandz+1
        print "zoom : dimension", zoomiim, zoomjjm, nlandz
        # print "zoom : nouvel index ",zoomkindex
        return(VarOutZoom, zoomkindex, zoomiim, zoomjjm, nlandz,
               invindex, (imin, imax, jmin, jmax))
    else:
        return(VarOutZoom)


# The input VarIn is a land variable.
# The output variable is 2D ( [jjm,iim] or [jmin:(jmax+1),imin:(imax+1)] )
# Simplified version of precedent one.
def zoom_land_simpl(VarIn, kindex, nbindex, zoomiim, zoomjjm, ifenetre, missing_value):
# where limits is in the order : [limit_W, limit_E, limit_N, limit_S]
    (imin, imax, jmin, jmax) = ifenetre
    VarOutZoom = missing_value*Numeric.ones((zoomjjm,zoomiim))
    for i in range(zoomiim):
        for j in range(zoomjjm):
            ki=kindex[imin+i,jmin+j]
            if (ki < missing_value):
                VarOutZoom[j,i] = VarIn[ki]
    return(VarOutZoom)


# The input VarIn is a 2D variable.
# the output variable is 2D ( [jjm,iim] or [jmin:(jmax+1),imin:(imax+1)] )
#            and if calcland==1, it returns too :
#                zoomkindex, zoomiim, zoomjjm, nlandz,
#                invindex, ifenetre =(imin,imax,jmin,jmax)
def zoom(VarIn, kindex, nbindex, lat, lon, iim, jjm, limits, missing_value, calcland=1):
# where limits is in the order : [limit_W, limit_E, limit_N, limit_S]
    imin = iim+1
    imax = -1
    jmin = jjm+1
    jmax = -1
    for j in range(jjm):
	for i in range(iim):
		if (kindex[i,j] < missing_value):
			if ( limits[0] <= lon[j,i] and lon[j,i] <= limits[1]
			     and limits[3] <= lat[j,i] and lat[j,i] <= limits[2] ):
				imin = min(imin, i)
				imax = max(imax, i)
				jmin = min(jmin, j)
				jmax = max(jmax, j)
                                invindex[kindex[i,j],0]=j
                                invindex[kindex[i,j],1]=i
    zoomiim = imax - imin +1
    zoomjjm = jmax - jmin +1
    # pourquoi le +1 ???? (python ??) => ncessaire en tout cas.
    #  cause des lat/lon qui doivent anglober la zone zoome
    VarOutZoom = VarIn[jmin:(jmax+1),imin:(imax+1)]
    if (calcland == 1):
        zoomkindex = Numeric.ones((zoomiim,zoomjjm))*int(missing_value)
        nlandz=0
        for j in range(jjm):
            for i in range(iim):
		if (kindex[i,j] < missing_value):
                    if ( limits[0] <= lon[j,i] and lon[j,i] <= limits[1]
                         and limits[3] <= lat[j,i] and lat[j,i] <= limits[2] ):
                        zoomkindex[i-imin, j-jmin] = kindex[i,j]
                        nlandz=nlandz+1
        print "zoom : dimension",zoomiim, zoomjjm, nlandz
        # print "zoom : nouvel index ",zoomkindex
        return(VarOutZoom, zoomkindex, zoomiim, zoomjjm, nlandz,
               invindex, (imin, imax, jmin, jmax))
    else:
        return(VarOutZoom)

# The input VarIn is a 2D variable.
# the output variable is 2D [jmin:(jmax+1),imin:(imax+1)]
def zoom2D(VarIn, lat, lon, iim, jjm, limits, missing_value):
# where limits is in the order : [limit_W, limit_E, limit_N, limit_S]
    imin = iim+1
    imax = -1
    jmin = jjm+1
    jmax = -1
    for j in range(jjm):
	for i in range(iim):
            if ( limits[0] <= lon[j,i] and lon[j,i] <= limits[1]
                 and limits[3] <= lat[j,i] and lat[j,i] <= limits[2] ):
                imin = min(imin, i)
                imax = max(imax, i)
                jmin = min(jmin, j)
                jmax = max(jmax, j)
    zoomiim = imax - imin +1
    zoomjjm = jmax - jmin +1
    VarOutZoom = VarIn[jmin:(jmax+1),imin:(imax+1)]
    return(VarOutZoom)

# The input VarIn is a 2D variable.
# the output variable is 2D ( [jjm,iim] or [jmin:(jmax+1),imin:(imax+1)] )
# Simplified version of precedent one.
def zoom_simpl(VarIn, zoomiim, zoomjjm, ifenetre, missing_value):
# where limits is in the order : [limit_W, limit_E, limit_N, limit_S]
    (imin, imax, jmin, jmax) = ifenetre
    VarOutZoom = missing_value*Numeric.ones((zoomjjm,zoomiim))
    VarOutZoom = VarIn[jmin:(jmax+1),imin:(imax+1)]
    return(VarOutZoom)


tab_mois=[31,28,31,30,31,30,31,31,30,31,30,31]

# Conversion Month/Day => Num of Day in year
def monthday_to_numday(month,day):
#Tableau definissant la duree des mois dans un calendrier de 365 jours
    numday=sum(tab_mois[0:(month-1)])+day
    return(numday)
    
# Rajoute dans le fichier "fichier" les tableaux bounds_lat et bounds_lon
# pour le rendre compatible avec la convention CF.

# def rajoutBounds(fichier,lat,lon,nland,land,kindex,nbindex,missing_value)
#     bounds_lon = Numeric.zeros((nland,,4))
#     bounds_lat = Numeric.zeros((nland,,4))
#     for j in range(jjm):
# 	for i in range(iim):
# 		if (kindex[i,j] < missing_vlaue):
#                         #print i,j,kindex[i,j],(jjm-1)-j
# 			VarOut[i,j] = VarIn[kindex[i,j]]
#     return(VarOut)


# Construction des tableaux de coordonnes 2D nav_lat et nav_lon
def Construit_nav_l(ax_lat, ax_lon, missing_v):
    Mones=Numeric.ones((len(ax_lat),1))
    Slon=Numeric.array([ax_lon])
    nvlon=MV.dot(Mones,Slon)
    nav_lon=cdms.createVariable(nvlon, 
                                typecode = cdms.MV.Float16,
                                fill_value = missing_v,
                                attributes={'name': 'nav_lon',
                                            'valid_min': -180.,
                                            'valid_max': 180.,
                                            'long_name': 'Longitude',
                                            'units': 'degrees_east' },
                                axes = [ ax_lat, ax_lon ],
                                id = 'nav_lon')

    Mones=Numeric.swapaxes(Numeric.ones((len(ax_lon),1)),0,1)
    tmplat=Numeric.swapaxes(Numeric.array([ax_lat]),0,1)
    nvlat=MV.dot(tmplat, Mones)
    nav_lat=cdms.createVariable(nvlat, 
                                typecode = cdms.MV.Float16,
                                fill_value = missing_v,
                                attributes={'name': 'nav_lat',
                                            'valid_min': -90.,
                                            'valid_max': 90.,
                                            'long_name': 'Latitude',
                                            'units': 'degrees_north' },
                                axes = [ ax_lat, ax_lon ],
                                id = 'nav_lat')
    return (nav_lat, nav_lon)
